/*
  Project: NEO-6M GY-GPS6MV2 GPS module, SD card or Micro SD card
  Function: Reads GPS raw data in NMEA strings. This sketch listen to the GPS serial port,
  and when data received from the module, it's also sent to the serial monitor and saved to SD card or Micro SD card.

  Wiring GPS module:
  GPS module -> Arduino Uno
  GND           GND
  RX pin        Digital pin 3
  TX pin        Digital pin 4
  VCC pin       5V

  Wiring SD card module:
  SD card module -> Arduino Uno
  VCC           3.3V or 5V (check module’s datasheet)
  CS            Digital pin 10. This can be the hardware SS pin - pin 10 (on most Arduino boards) or pin 53 (on the Mega) - or another pin specified in the call to SD.begin(). N
              ote that even if you don't use the hardware SS pin, it must be left as an output or the SD library won't work. Different boards use different
              pins for this functionality, so be sure you’ve selected the correct pin in SD.begin().
  MOSI          Digital pin 11
  CLK           Digital pin 13
  MISO          Digital pin 12
  GND           GND

  Note: different Arduino boards have different SPI pins. If you’re using another Arduino board, check the Arduino SPI documentation.
*/
//************************************************************
#include <SoftwareSerial.h>//include library code to allow serial communication on other digital pins of the Arduino board
#include <SPI.h>//include library code to communicate with SPI devices
#include <SD.h>//include library code for SD card
//************************************************************
const int chipSelect = 10;
File dataFile;
SoftwareSerial GPSModule(4, 3); // RX, TX

// process and flow related variables
int updates;
int failedUpdates;
int pos;
int stringplace = 0;
//
// variable to hold out nmea data and labels.
String nmea[12];
String labels[12] {"Time: ", "Lat: ", "N/S: ", "Lng: ", "E/W: ", "Fix: ", "Sats: ", "HDP: ", "Alt: ", "Metres: ", "MSL: ", "Metres: "};
//
//****************************************************************************
void setup() {
  Serial.begin(9600);
  GPSModule.begin(9600);
  Serial.print("Initializing SD card...");//setup for the SD card
  if (!SD.begin(chipSelect)) {
    Serial.println("Card failed, or not present!");
    return;
  }
  Serial.println("Card initialized.");
  dataFile = SD.open("gps.txt", FILE_WRITE); //opens file
  if (dataFile) { //if the file opened ok, writes to it
    dataFile.println("Line number, Time , Lat , N/S, Long , E/W , Sats , HDP , Alt , MSL ");//prints the headings for our data
  }
  dataFile.close();//file closed
}

void loop() {
  dataFile = SD.open("gps.txt", FILE_WRITE);//opens file
  //flushing out data at the start of each loop, not sure it is really neccessay
  Serial.flush();
  GPSModule.flush();
  while (GPSModule.available() > 0)
  {
    GPSModule.read();
  }
  // This is the meat of it really, we look for our nmea string and then split it into comma seperated values
  if (GPSModule.find("$GPGGA,")) {
    String tempMsg = GPSModule.readStringUntil('\n');
    //Serial.println(tempMsg);
    for (int i = 0; i < tempMsg.length(); i++) {
      if (tempMsg.substring(i, i + 1) == ",") {
        nmea[pos] = tempMsg.substring(stringplace, i);
        stringplace = i + 1;
        pos++;
      }
      if (i == tempMsg.length() - 1) {
        nmea[pos] = tempMsg.substring(stringplace, i);
      }
    }
    updates++;

    // Here we convert our minutes and seconds latitute and longitute to decimal degrees
    nmea[1] = ConvertLat();
    nmea[3] = ConvertLng();

    // Test to see if GPS is fixed
    if (nmea[5].toInt() > 0) {

      // save data to SD if data is fixed
      if (dataFile) {
        Serial.println("File opened. Start logging GPS data to SD card");
        // update number
        dataFile.print(updates);
        dataFile.print(",");
        // time
        dataFile.print(nmea[0]);
        dataFile.print(",");
        // latitude
        dataFile.print(nmea[1]);
        dataFile.print(",");
        // N/S
        dataFile.print(nmea[2]);
        dataFile.print(",");
        // longitude
        dataFile.print(nmea[3]);
        dataFile.print(",");
        // E/W
        dataFile.print(nmea[4]);
        dataFile.print(",");
        // number of satellites
        dataFile.print(nmea[6]);
        dataFile.print(",");
        // HDP
        dataFile.print(nmea[7]);
        dataFile.print(",");
        // Altitude, meters
        dataFile.print(nmea[8]);
        dataFile.print(",");
        // MSL, meters
        dataFile.print(nmea[10]);
        dataFile.println("");
        
      }
      else {
        Serial.println("Error. Problem with logging GPS data to SD card");
      }
    }
  }
  else {
    failedUpdates++;
    Serial.println(failedUpdates);
  }

  // Print out data to the Serial Monitor
  for (int i = 0; i < 11; i++) {
    Serial.print(labels[i]);
    Serial.print(nmea[i]);
    Serial.println("");
  }

  // Zero our flow control varibles
  stringplace = 0;
  pos = 0;

  dataFile.close();// close file on SD card
}
//******************************************************************
String ConvertLat() {
  String posneg = "";
  if (nmea[2] == "S") {
    posneg = "-";
  }
  String latfirst;
  float latsecond;
  for (int i = 0; i < nmea[1].length(); i++) {
    if (nmea[1].substring(i, i + 1) == ".") {
      latfirst = nmea[1].substring(0, i - 2);
      latsecond = nmea[1].substring(i - 2).toFloat();
    }
  }
  latsecond = latsecond / 60;
  String CalcLat = "";

  char charVal[9];
  dtostrf(latsecond, 4, 6, charVal);
  for (int i = 0; i < sizeof(charVal); i++)
  {
    CalcLat += charVal[i];
  }
  latfirst += CalcLat.substring(1);
  latfirst = posneg += latfirst;
  return latfirst;
}
//************************************************************************
String ConvertLng() {
  String posneg = "";
  if (nmea[4] == "W") {
    posneg = "-";
  }

  String lngfirst;
  float lngsecond;
  for (int i = 0; i < nmea[3].length(); i++) {
    if (nmea[3].substring(i, i + 1) == ".") {
      lngfirst = nmea[3].substring(0, i - 2);
      //Serial.println(lngfirst);
      lngsecond = nmea[3].substring(i - 2).toFloat();
      //Serial.println(lngsecond);

    }
  }
  lngsecond = lngsecond / 60;
  String CalcLng = "";
  char charVal[9];
  dtostrf(lngsecond, 4, 6, charVal);
  for (int i = 0; i < sizeof(charVal); i++)
  {
    CalcLng += charVal[i];
  }
  lngfirst += CalcLng.substring(1);
  lngfirst = posneg += lngfirst;
  return lngfirst;
}



